package edu.iastate.jrelm.rl.rotherev.advanced;

import edu.iastate.jrelm.core.Action;
import edu.iastate.jrelm.similarity.DissimilarityMeasure;
import edu.iastate.jrelm.spillover.SpilloverByDissimilarity;
import edu.iastate.jrelm.spillover.SpilloverWeightingException;

 /** 
 * Implements a simple exponential weight function of the form 
 * 
 * 			e^(b * x)
 * 
 * where x is a Double value indicating the dissimilarity of two Actions as 
 * determined by the given DissimilarityMeasure. The parameter 'b' can set
 * to alter the shape of the exponential curve. 
 * <p>
 * Note: The VRELearner uses weight values between 0 and 1. As such, 'b' 
 * must be set to a value <= 0.
 * 
 *  
 * @author Charles Gieseler
 *
 * @param <A> - The type of Action 
 * @param <D> - The type of DissimilarityMeasure to use.
 * <p>
 * Note: D is restricted to DissimilarityMeasures 
 * that return Doubles as indicators.
 */
public class ExponentialSpillover
		<A extends Action, 
		 D extends DissimilarityMeasure<A, Double>> 
		extends SpilloverByDissimilarity<A,D> {

	double paramB = -1;
	
	/**
	 * Construct an exponential weighing method with the given DissimilarityMeasure 
	 * and parameter value.
	 * 
	 * @param measure - the DissimilarityMeasure to use for comparison of Actions
	 * @param b - parameter for adjusting the shape of the exponential curve
	 */
	public ExponentialSpillover(D measure, double b){
		super(measure);
		
		try{
			this.setParamB(b);
		}catch(SpilloverWeightingException swe){
			System.err.println("Received an illegal value for parameter 'b' " +
					"in ExponentialSpillover. Using default value of -1.");
			System.err.println(swe.getMessage());
		}
	}
	
	/**
	 * Generates a reward spillover weight using an exponential function
	 * 
	 * 			e^(b * x)
	 * 
	 * where x is a numerical (Double) indicator of how dissimilar the given Action
	 * is to the reference Action. 
	 */
	public double generateWeight(A action) {
		double weight = 0;
		Double dissimilarityValue;
		double x;
		
		
		//Compare the given Action with the reference Action to
		//determine how similar they are
		A refAction = getReferenceAction();
		dissimilarityValue = getDissimilarityMeasure().dissimilarity(refAction, action);
		x = dissimilarityValue.doubleValue();
		
		//e^(b * x)
		weight = Math.exp(x);
		
		return weight;
	}
	
	public String getName(){return "Exponential Weighting";}
	
	/**
	 * 
	 * @return the value of parameter 'b'
	 */
	public double getParamB() {
		return paramB;
	}
	
	/**
	 * Set the value of the 'b' parameter.
	 * 
	 * Note: The VRELearner uses weight values between 0 and 1. As such, 'b' 
	 * must be set to a value <= 0.
	 * 
	 * @param bValue - desired value for parameter 'b'
	 */
	public void setParamB(double bValue) throws SpilloverWeightingException {
		if(bValue <= 0)
			this.paramB = bValue;
		else
			throw new SpilloverWeightingException("Illegal value for parameter 'b' in " +
					"RothErevExonentialWeighting: " + bValue +". Must set 'b' to a value <= 0.");
	}
}
